#ifndef CPPUT_IMPL_ASSERTIONS_H_INCLUDED
# define CPPUT_IMPL_ASSERTIONS_H_INCLUDED

// This header must never be included directly. It is an implementation details of
// <cpput/testing.h>. Include <cpput/testing.h> instead.
# ifndef CPPUT_TESTING_H_PROLOG_IMPL_INCLUDES
#  error This header must never be included directly. Includes <cpput/testing.h> instead.
# endif

namespace CppUT { namespace Impl {

   /** Unique type per comparison operator
    * The argument value is derived from the following table,
    * where op is the comparison operator:
    * 1 op 2: true for operator <, <=, !=;  if true then add 1 to value
    * 2 op 1: true for operator >, >=, !=;  if true then add 2 to value
    * 0 op 0: true for operator <=, ==, >=; if true then add 4 to value
    */
   template<const int>
   struct Operator
   {
   };

   typedef Operator<1> LesserThan;
   typedef Operator<1+4> LesserOrEqual;
   typedef Operator<4> Equal;
   typedef Operator<1+2> NotEqual;
   typedef Operator<2+4> GreaterOrEqual;
   typedef Operator<2> GreaterThan;

   } // namespace Impl
} // namespace CppUT

/** Returns the Operator as a C++ type.
 * Example: _CPPUT_COMPARISON_OPERATOR_TYPE( < ) returns type LesserThan.
 */
# define _CPPUT_COMPARISON_OPERATOR_TYPE( op ) \
   ::CppUT::Impl::Operator< (1 op 2 ? 1:0) + (2 op 1 ? 2:0) + (0 op 0 ? 4:0) >

/** Returns the reverse Operator as a C++ type.
 * Returns type:
 * _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( < ): GreaterThan
 * _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( <= ): GreaterOrEqual
 * _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( == ): Equal
 * _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( != ): NotEqual
 * _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( >= ): LesserOrEqual
 * _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( > ): LesserThan
 */
# define _CPPUT_COMPARISON_REVERSE_OPERATOR_TYPE( op ) \
   ::CppUT::Impl::Operator< (1 op 2 ? 2:0) + (2 op 1 ? 1:0) + (0 op 0 ? 4:0) >


#endif // CPPUT_IMPL_ASSERTIONS_H_INCLUDED
