// Defines the convertToString() & stringize() function use by assertions to output a value.
#ifndef CPPUT_IMPL_STRINGIZE_H_INCLUDED
# define CPPUT_IMPL_STRINGIZE_H_INCLUDED

# include <cpput/config.h>
# include <string>
# include <cpptl/stringtools.h>


/// \cond implementation_detail

namespace CppUT { namespace Impl {
   struct NoToStringOverload {};

   /*! \ingroup group_assertions_support
    */
   struct NotConvertibleToStdString {};

   template<typename ValueType>
   std::string toStringStringize( const ValueType &value, NoToStringOverload )
   {
      return defaultStringize( value );
   }

   template<typename ValueType>
   std::string toStringStringize( const ValueType &value, const std::string &str )
   {
      return str;
   }

   template<typename ValueType>
   std::string stringize( const ValueType &value, NotConvertibleToStdString )
   {
      return toStringStringize( value, toString( value ) );
   }

   template<typename ValueType>
   std::string stringize( const ValueType &value, const std::string &str )
   {
      return CppTL::quoteString( str ).c_str();
   }

}   } // namespace CppUT { namespace Impl

/// \endcond implementation_detail

#endif // CPPUT_IMPL_STRINGIZE_H_INCLUDED
