#ifndef CPPUT_IMPL_TESTCASE_H_INCLUDED
# define CPPUT_IMPL_TESTCASE_H_INCLUDED

// This header must never be included directly. It is an implementation details of
// <cpput/testing.h>. Include <cpput/testing.h> instead.
# ifndef CPPUT_TESTING_H_PROLOG_IMPL_INCLUDES
#  error This header must never be included directly. Includes <cpput/testing.h> instead.
# endif

namespace CppUT {
   namespace Impl {

// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////
// Abstract/Polymorphic test fixture implementation macros
// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////

// Internal
// Prolog parts that is common to abstract, base and derived fixture classes.
// Declares test case factory and run member function pointer.
#define _CPPUT_TESTSUITE_COMMON( FixtureType )                             \
   public:                                                                 \
   void (FixtureType :: *cpputRun_)();                                     \
                                                                           \
   typedef FixtureType CppUTSelfType;                                      \
                                                                           \
   struct FixtureTestFactory                                               \
   {                                                                       \
      typedef ::CppUT::TestCase *result_type;                              \
                                                                           \
      void (FixtureType :: *run_)();                                       \
      ::CppUT::TestCaseFactoryFn factory_;                                 \
                                                                           \
                                                                           \
      static void addTest( ::CppUT::Suite &suite,                          \
         ::CppUT::TestCaseFactoryFn fixtureFactory,                        \
         const char *name,                                                 \
         void (FixtureType :: *run)() )                                    \
      {                                                                    \
         FixtureTestFactory factory;                                       \
         factory.run_ = run;                                               \
         factory.factory_ = fixtureFactory;                                \
         suite.add( ::CppUT::TestMeta( ::CppTL::fn0r( factory ),           \
                                       name ) );                           \
      }                                                                    \
                                                                           \
      static void addTest( ::CppUT::Suite &suite,                          \
         ::CppUT::TestCaseFactoryFn fixtureFactory,                        \
         const char *name,                                                 \
         void (FixtureType :: *run)(),                                     \
         const ::CppUT::TestExtendedData &metaData )                       \
      {                                                                    \
         FixtureTestFactory factory;                                       \
         factory.run_ = run;                                               \
         factory.factory_ = fixtureFactory;                                \
         suite.add( ::CppUT::TestMeta( ::CppTL::fn0r( factory ),           \
                                       ::CppUT::MetaData( name, metaData ) \
                                     ) );                                  \
      }                                                                    \
                                                                           \
                                                                           \
      ::CppUT::TestCase *operator()() const                                \
      {                                                                    \
         FixtureType *fixture = static_cast<FixtureType *>( factory_() );  \
         if ( fixture != 0 )                                               \
         {                                                                 \
            fixture->cpputClearRunnable();                                 \
            fixture->cpputRun_ = run_;                                     \
         }                                                                 \
         return fixture;                                                   \
      }                                                                    \
   };                                                                      \
                                                                           \
                                                                           \
   virtual void cpputClearRunnable()                                       \
   {                                                                       \
      cpputRun_ = 0;                                                       \
   }


// Internal
// Part that is common to all concrete fixture classes
// Declares fixture, and suite factories.
#define _CPPUT_TESTSUITE_COMMON_CONCRETE( FixtureType )                    \
   static ::CppUT::TestCase *fixtureFactory()                              \
   {                                                                       \
      return new FixtureType();                                            \
   }                                                                       \
                                                                           \
   static ::CppUT::Suite suite()                                           \
   {                                                                       \
      ::CppUT::Suite thisSuite( #FixtureType );                            \
      addTests( thisSuite );                                               \
      return thisSuite;                                                    \
   }                                                                       \
                                                                           \
   static void addTests( ::CppUT::Suite &suite,                            \
      ::CppUT::TestCaseFactoryFn factory = &FixtureType :: fixtureFactory )\
   {

// Internal
// Part that is common to all abstract fixture classes.
// Declares suite factory.
# define _CPPUT_TESTSUITE_COMMON_ABSTRACT( FixtureType )                   \
   static void addTests( ::CppUT::Suite &suite,                            \
                         ::CppUT::TestCaseFactoryFn factory )              \
   {


// Internal
// Part that is common to all derived fixture classes.
// Chaine suite factory with parent class factory.
# define _CPPUT_TESTSUITE_COMMON_EXTEND( FixtureType, ParentFixtureType )  \
      ParentFixtureType :: addTests( suite, factory )


// Internal
// Part that is common to all base hierarchy fixture classes.
// Run test based on content of member function pointer cpputRun_.
# define _CPPUT_TESTSUITE_BASE_RUN( FixtureType )                          \
   /* overridden from CppUT::TestCase */                                   \
   virtual void run()                                                      \
   {                                                                       \
      (this->*cpputRun_)();                                                \
   }


// Internal
// Part that is common to all derived fixture classes.
// Run test based on content of member function pointer cpputRun_ if set,
// otherwise forward run call to base class.
# define _CPPUT_TESTSUITE_DERIVED_RUN( FixtureType, ParentFixtureType )    \
   /* overridden from CppUT::TestCase */                                   \
   virtual void run()                                                      \
   {                                                                       \
      if ( cpputRun_ == 0 )                                                \
         ParentFixtureType :: run();                                       \
      else                                                                 \
         (this->*cpputRun_)();                                             \
   }




#if 0


# if CPPUT_USE_RTTI_TO_NAME_SUITE
#  define CPPUT_TESTSUITE_NAME_FROM_TYPE( FixtureType )					\
      ::CppTL::getTypeName( ::CppTL::Type<FixtureType>(), #FixtureType )
# else // if CPPUT_USE_RTTI_TO_NAME_SUITE
#  define CPPUT_TESTSUITE_NAME_FROM_TYPE( FixtureType )					\
      std::string( #FixtureType )
# endif // if CPPUT_USE_RTTI_TO_NAME_SUITE

#endif


// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////
// Lightweight test fixture
// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////
      TestMeta CPPUT_API 
      declareFixtureTestCase( TestCaseFactoryFn factory,
                              const char *fixtureSuiteName,
                              const char *testCaseName,
                              Suite parentSuite );

      TestMeta CPPUT_API 
      declareFixtureTestCase( TestCaseFactoryFn factory,
                              const char *fixtureSuiteName,
                              const char *testCaseName,
                              const ::CppUT::TestExtendedData &metaData,
                              Suite parentSuite );

#define _CPPUT_FIXTURE_TEST_COMMON( FixtureType, testFunction )         \
   class FixtureType##testFunction : public FixtureType                 \
   {                                                                    \
      static ::CppUT::TestMeta testMeta;                                \
   public:                                                              \
      static ::CppUT::TestCase *cpputFactory()                          \
      {                                                                 \
         return new FixtureType##testFunction();                        \
      }                                                                 \
                                                                        \
   public: /* overriden from CppUT::TestCase */                         \
      virtual void run();                                               \
   };                                                                   \
                                                                        \
   ::CppUT::TestMeta FixtureType##testFunction::testMeta(               \
      ::CppUT::Impl::declareFixtureTestCase(                            \
                              &FixtureType##testFunction::cpputFactory, \
                              #FixtureType,                             \
                              #testFunction,                            \



// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////
// Free function based test cases
// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////

   TestMeta CPPUT_API
   declareTestCaseFunction( void (*run)(), 
                            const char *name, 
                            Suite parentSuite );

   TestMeta CPPUT_API 
   declareTestCaseFunction( void (*run)(), 
                            const MetaData &metaData,
                            Suite parentSuite );

/*! \brief Make the specified C function a test case and register it in the current suite.
 * \code
 * static void myTest1() {
 *   CPPUT_ASSERT_TRUE( false );
 * }
 * CPPUT_REGISTER_TEST_FUNCTION( myTest1 );
 * \endcode
 */
#define _CPPUT_REGISTER_TEST_FUNCTION( testFunction )                      \
   static ::CppUT::TestMeta                                                \
      CPPTL_MAKE_UNIQUE_NAME( cpputmeta ## testFunctionName ) =            \
      ::CppUT::Impl::declareTestCaseFunction( &testFunction,               \
                                              #testFunction,               \
                                              CPPUT_CURRENT_SUITE() )

#define _CPPUT_REGISTER_TEST_FUNCTION_WITH_META( testFunction, metaData )  \
   static ::CppUT::TestMeta                                                \
      CPPTL_MAKE_UNIQUE_NAME( cpputmeta ## testFunctionName ) =            \
      ::CppUT::Impl::declareTestCaseFunction( &testFunction,               \
                                              metaData,                    \
                                              CPPUT_CURRENT_SUITE() )



// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////
// Test Suite declarations
// //////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////

      /// Creates a suite with the specified name and makes it the new default root suite.
      /// The suite is nested into the current default root suite if any,
      /// or the top level root suite otherwise.
      /// @return Always \c NULL.
      StaticSuite *makeAndSetDefaultRootSuite( const char *name );

      /// Sets the default root suite to the top level root suite.
      /// @return Always \c NULL.
      StaticSuite *resetDefaultRootSuite();

      /// Creates a suite with the specified name.
      /// The suite is nested into the current default suite if any,
      /// or the top level root suite otherwise.
      /// @return Always \c NULL.
      StaticSuite *makeNestedSuiteInDefaultRootSuite( const char *name );

      /// Makes \c suite a child of \c parentSuite.
      /// \c suite is removed from its current parent and make
      /// a nested suite of \c parentSuite.
      /// @return Always \c NULL.
      StaticSuite *reparentSuite( Suite parentSuite, 
                                  Suite suite );


      SuiteMeta currentSuite(); 

// internal
#define _CPPUT_DECLARE_ROOT_SUITE( name, factory )                \
   static ::CppUT::StaticSuite *                                  \
      CPPTL_MAKE_UNIQUE_NAME(cpputRootSuite) =                    \
      ::CppUT::Impl::factory( name )


   } // namespace Impl
} // namespace CppUT


namespace {
   // Cause the static initializer to be called in each translation unit,
   // Reset the current suite to the top root suite.
   static ::CppUT::StaticSuite *cpputTopRootSuite = 
      ::CppUT::Impl::resetDefaultRootSuite();
} // end anonymous namespace


#endif // CPPUT_IMPL_TESTCASE_H_INCLUDED
