#ifndef CPPUT_IMPL_TESTCASEEPILOG_H_INCLUDED
# define CPPUT_IMPL_TESTCASEEPILOG_H_INCLUDED

// This header must never be included directly. It is an implementation details of
// <cpput/testing.h>. Include <cpput/testing.h> instead.
# ifndef CPPUT_TESTING_H_EPILOG_IMPL_INCLUDES
#  error This header must never be included directly. Includes <cpput/testing.h> instead.
# endif

// Notes: class CppUT::TestExtendedData must have been defined before this header is included.

namespace CppUT {
   namespace Impl {


   /*! \ingroup group_testfixture
    */
   class CPPUT_API DefaultTestExtendedData : public TestExtendedData
   {
   public: // overridden from TestExtendedData
      void apply( MetaData &test ) const;
   };


   /*! \ingroup group_testfixture
    */
   class TestExtendedDataList : public TestExtendedData
   {
   public:
      TestExtendedDataList( const TestExtendedData &left,
                            const TestExtendedData &right );
   public: // overridden from TestExtendedData
      void apply( MetaData &test ) const;

   private:
      const TestExtendedData &left_;
      const TestExtendedData &right_;
   };


   /*! \ingroup group_testfixture
    */
   class CPPUT_API DescriptionData : public TestExtendedData
   {
   public:
      DescriptionData( const std::string &description );

   public: // overridden from TestExtendedData
      void apply( MetaData &test ) const;

   private:
      std::string description_;
   };


   /*! \ingroup group_testfixture
    */
   class CPPUT_API TimeOutData : public TestExtendedData
   {
   public:
      TimeOutData( double timeOutInSeconds );

   public: // overridden from TestExtendedData
      void apply( MetaData &test ) const;

   private:
      double timeOutInSeconds_;
   };


   /*! \ingroup group_testfixture
    */
   class CPPUT_API DependenciesData : public TestExtendedData
   {
   public:
      DependenciesData(  const std::string &dependencies  );

   public: // overridden from TestExtendedData
      void apply( MetaData &test ) const;
   private:
      std::string dependencies_;
   };


   /*! \ingroup group_testfixture
    */
   class CPPUT_API GroupData : public TestExtendedData
   {
   public:
      GroupData(  const std::string &groupName );

   public: // overridden from TestExtendedData
      void apply( MetaData &test ) const;
   private:
      std::string groupName_;
   };


   } // namespace Impl
} // namespace CppUT


#endif // CPPUT_IMPL_TESTCASEEPILOG_H_INCLUDED
