#ifndef CPPUTTOOLS_CONFIGURATION_VALUE_H_INCUDED
# define CPPUTTOOLS_CONFIGURATION_VALUE_H_INCUDED

# include "simplertti.h"
# include "validator.h"
# include <cpputtools/value.h>

namespace CppUTTools { namespace CommandLineOptions {


class OptionHandler
{
public:
   virtual ~OptionHandler() {}

   virtual Value validate( const Parameters &parameters ) const = 0;
};



template<class TheOptionType>
class SimpleOptionHandler : public OptionHandler
{
public:
   typedef TheOptionType OptionType;

   SimpleOptionHandler();

   SimpleOptionHandler( Validator *validator );

   SimpleOptionHandler( Validator *validator, OptionType *storage );

   ~SimpleOptionHandler();

public: // overridden from OptionHandler
   Value validate( const Parameters &parameters ) const;

private:
   Validator *validator_;
};



// template class SimpleOptionHandler
// //////////////////////////////////////////////////////////////////

template<class TheOptionType>
SimpleOptionHandler<TheOptionType>::SimpleOptionHandler()
   : storage_( 0 )
   , validator_( new StringStreamValidator<TheOptionType>() )
{
}

template<class TheOptionType>
SimpleOptionHandler<TheOptionType>::SimpleOptionHandler( OptionType *storage )
   : storage_( storage )
   , validator_( new StringStreamValidator<TheOptionType>() )
{
}


template<class TheOptionType>
SimpleOptionHandler<TheOptionType>::SimpleOptionHandler( Validator *validator )
   : storage_( 0 )
   , validator_( validator )
{
}


template<class TheOptionType>
SimpleOptionHandler<TheOptionType>::SimpleOptionHandler( Validator *validator, 
                                                         OptionType *storage )
   : storage_( storage )
   , validator_( validator )
{
}

template<class TheOptionType>
SimpleOptionHandler<TheOptionType>::~SimpleOptionHandler()
{
   delete validator_;
}


template<class TheOptionType>
bool 
SimpleOptionHandler<TheOptionType>::hasStorage() const
{
   return storage_ != 0;
}

template<class TheOptionType>
void 
SimpleOptionHandler<TheOptionType>::storeValue( const Value &value ) const
{
   if ( hasStorage() )
      *storage_ = get<TheOptionType>( value );
}


template<class TheOptionType>
Value
SimpleOptionHandler<TheOptionType>::validate( const Parameters &parameters ) const
{
   return validator_->validate( parameters );
}


} } // namespace CommandLineOptions

#endif // CPPUTTOOLS_CONFIGURATION_VALUE_H_INCUDED
