#include <cpput/ipc.h>
#include <stdio.h>
#include <string>
#include <deque>

   typedef std::deque<std::string> Lines;

static void
makeLines( std::string &input, Lines &lines )
{
   std::string::size_type pos = 0;
   std::string::size_type nextEOL = std::string::npos;
   while (true)
   {
      nextEOL = input.find( '\n', pos );
      if ( nextEOL == std::string::npos )
      {
         break;
      }
      std::string line = input.substr( pos, nextEOL - pos );
      if ( !line.empty()  &&  line[line.size()-1] == '\r' )
      {
         line.erase( line.size()-1 );
      }
      lines.push_back( line );

      pos = nextEOL + 1;
   }
   std::string newInput = input.substr( pos );
   input = newInput;
}
      

static void
process( Lines &lines, CppUT::SynchronousWriteChannel &channel )
{
   while ( !lines.empty() )
   {
      std::string line = lines.front();
      lines.pop_front();
      std::string text = "Received: ";
      text += line;
      text += "\n";
      if ( !channel.write( text.c_str(), text.size() ) )
      {
         fprintf( stderr, "Failed to write reply: '%s'\n", text.c_str() );
      }
   }
}


int main( int argc, const char *argv[] )
{

   CppUT::SynchronousWriteChannelAutoPtr channel = CppUT::makeSlaveChannel();

   fprintf( stderr, "Waiting for data\n" );

   Lines lines;
   std::string input;
   while ( true )
   {
      char buffer[8];
      size_t readLength;
      if ( !channel->read( &buffer, sizeof(buffer), readLength ) )
      {
         break;
      }

      input.append( buffer, buffer + readLength );
      makeLines( input, lines );
      process( lines, *channel );
   }

   if ( !input.empty() )
   {
      lines.push_back( input );
      process( lines, *channel );
   }

   fprintf( stderr, "Failed to read data from input\n" );

   channel->close();

   return 0;
}